using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using CsvHelper;
using CsvHelper.Configuration;

namespace eea_e1a_get
{

    public class WebClientWithTimeout : WebClient
    {
        protected override WebRequest GetWebRequest(Uri address)
        {
            WebRequest wr = base.GetWebRequest(address);
            wr.Timeout = 1000 * 60 * 20; // timeout in milliseconds (ms)
            return wr;
        }
    }

    public static class EEAService
    {
        private static string EEA_URL = "https://fme.discomap.eea.europa.eu/fmedatastreaming/AirQualityDownload/AQData_Extract.fmw?CountryCode={0}&CityName={1}&Pollutant={2}&Year_from={3}&Year_to={3}&Station=&Samplingpoint=&Source={4}&Output=TEXT&UpdateDate=";
        private static string METADATA_URL = "https://discomap.eea.europa.eu/map/fme/metadata/PanEuropean_metadata.csv";
        private static HttpClient Client = new HttpClient();
        // private static Configuration ConfigMeta = new Configuration
        // {
        //     HasHeaderRecord = true,
        //     HeaderValidated = null,
        //     MissingFieldFound = null,
        //     IgnoreBlankLines = true,
        //     Delimiter = "\t",
        // };
        // private static Configuration Config = new Configuration
        // {
        //     HasHeaderRecord = true,
        //     HeaderValidated = null,
        //     MissingFieldFound = null,
        //     IgnoreBlankLines = true,
        // };

        public static string[] GetDatasources(string country, string city, string source, int year, int pollutantId)
        {
            string eea_url = String.Format(EEA_URL, country, city, pollutantId, year, source);
            WebClientWithTimeout httpClient = new WebClientWithTimeout();
            var str = httpClient.DownloadString(eea_url);
            var datasource_urls = str.Split(new[] { "\r\n", "\r", "\n" }, StringSplitOptions.RemoveEmptyEntries);
            return datasource_urls;
        }

        public static async Task<List<E1A>> GetE1A(string url)
        {
            using (var stream = await Client.GetStreamAsync(url))
            using (var reader = new StreamReader(stream))
            using (var csv = new CsvReader(reader, CultureInfo.InvariantCulture))
            {
                csv.Configuration.HasHeaderRecord = true;
                csv.Configuration.HeaderValidated = null;
                csv.Configuration.MissingFieldFound = null;
                csv.Configuration.IgnoreBlankLines = true;
                var records = csv.GetRecords<E1A>().ToList();
                string resolution = records.First().AveragingTime;
                /* bool hasSameAllowedResolution = records.Count == records.Count(p=>p.AveragingTime == resolution && Resolution.AllowedResolutions.Contains(p.AveragingTime));
                if(!hasSameAllowedResolution)
                    throw new Exception("Dataset was either irregular or did not contain an allowed resolution");
                */

                var validRecords = records.Where(p => p.Validity > 0).ToList();
                if (validRecords.Count == 0)
                    throw new Exception("Dataset did not contain any valid values");

                return validRecords;
            }
        }

        public static async Task<List<Metadata>> GetMetadata()
        {
            List<Metadata> metadata = new List<Metadata>();
            using (var stream = await Client.GetStreamAsync(METADATA_URL))
            using (var reader = new StreamReader(stream))
            using (var csv = new CsvReader(reader, CultureInfo.InvariantCulture))
            {
                csv.Configuration.Delimiter = "\t";
                while (csv.Read())
                {
                    try
                    {
                        var record = csv.GetRecord<Metadata>();
                        metadata.Add(record);
                    }
                    catch (System.Exception e) { }
                }
            }
            return metadata;
        }
    }
}