using System;
using System.Collections.Generic;

namespace eea_e1a_get
{
    public class Resolution
    {
        public static List<string> AllowedResolutions = new List<string>() {"hour", "day"};

        public readonly string ResolutionAsString;

        public Resolution(string resolution)
        {
            if(!AllowedResolutions.Contains(resolution))
                throw new Exception(resolution + " resolution not supported");
            ResolutionAsString = resolution;
        }

        public int GetObservationsInAYear (int year) 
        {
            int offset = DateTime.IsLeapYear(year) ? 1 : 0;            
            return ( 365 + offset ) * (ResolutionAsString == "hour" ? 24 : 1);
        }

        public DateTime AddTimeStep(DateTime dt, int step)
        {
            // +1 hour to get totime
            return ResolutionAsString == "hour" ? dt.AddHours(step+1) : dt.AddDays(step);
        }

        public DateTime ToUniversalTime(DateTime dt)
        {    
            // Ignore UTC for day. Convert to fromtime instead        
            var d = ResolutionAsString == "hour" ? dt.ToUniversalTime() : dt.AddDays(-1);
            d = ResolutionAsString == "hour" ? d : new DateTime(d.Year, d.Month, d.Day);
            return d;
        }
    }
}